package com.nisum.ecomservice.controller;

import com.nisum.ecomservice.dto.CartDTO;
import com.nisum.ecomservice.model.CartItemEntity;
import com.nisum.ecomservice.repository.CartRepository;
import com.nisum.ecomservice.service.CartService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;


@RestController
@CrossOrigin(origins = "*")
public class CartController {
    @Autowired
    private CartService cartService;

    @Autowired
    private CartRepository cartRepository;

    @GetMapping("/api/carts")
    public ResponseEntity<Flux<CartDTO>>getAllUserCarts(){
        return ResponseEntity.ok(cartService.getAllUsersCartItems());
    }

    @GetMapping("/api/carts/{userId}")
    public ResponseEntity<Mono<CartDTO>>getCartByUserId(@PathVariable String userId) {
        Mono<CartDTO> cartDto= cartService.getUserIdCartItems(userId);
        HttpStatus status = cartDto != null ? HttpStatus.OK : HttpStatus.NOT_FOUND;
        return new ResponseEntity<>(cartDto, status);
    }

    @PostMapping("/api/carts")
    @ResponseStatus(HttpStatus.CREATED)
    public Mono<CartDTO>addToCartByUserId(@RequestBody CartDTO cartDTO) {
        return cartService.addUserIdItemsToCart(cartDTO);
    }

    @PutMapping("/api/carts/{userId}")
    public Mono<ResponseEntity<CartDTO>>updateCartByUserId(@PathVariable String userId, @RequestBody CartDTO cartDTO){
        return cartService.updateUserIdCartItems(userId, cartDTO)
                .map(updatedItem->ResponseEntity.ok(updatedItem))
                .defaultIfEmpty(ResponseEntity.badRequest().build());
    }

    @DeleteMapping("/api/carts/{userId}")
    public Mono<ResponseEntity<Void>>deleteCartByUserId(@PathVariable String userId){
        return cartService.deleteUserIdCartItems(userId)
                .map(res-> ResponseEntity.ok().<Void>build())
                .defaultIfEmpty(ResponseEntity.notFound().build());
    }
}