package com.nisum.ecomservice.service;

import com.nisum.ecomservice.dto.CartDTO;
import com.nisum.ecomservice.dto.CartItemDTO;
import com.nisum.ecomservice.exception.ResourceNotFoundException;
import com.nisum.ecomservice.model.CartEntity;
import com.nisum.ecomservice.model.CartItemEntity;
import com.nisum.ecomservice.repository.CartRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class CartService {

    @Autowired
    private CartRepository cartRepository;


    public Flux<CartDTO> getAllUsersCartItems() {
        return cartRepository.findAll().map(cartItems->convertEntitytoDTO(cartItems));
    }

    public Mono<CartDTO> getUserIdCartItems(String userId){
        return cartRepository.findById(userId)
                .map(existingItems->convertEntitytoDTO(existingItems))
                .switchIfEmpty(Mono.error(new ResourceNotFoundException(HttpStatus.NOT_FOUND,
                        "CartItem for UserId" + userId + "not found")));
    }

    public Mono<CartDTO>addUserIdItemsToCart(CartDTO cartDTO) {
        cartRepository.deleteById(cartDTO.getUserId());
        Mono<CartEntity>  cartEntityDB=  cartRepository.save(convertDTOtoEntity(cartDTO));
        return   cartEntityDB.map(newItem->convertEntitytoDTO(newItem));
    }

    public Mono<CartDTO>updateUserIdCartItems(String userId, CartDTO cartDTO) {
        Mono<CartEntity>  cartEntityDB=  cartRepository.save(convertDTOtoEntity(cartDTO));
        return   cartEntityDB.map(updatedItem->convertEntitytoDTO(updatedItem));
    }

    public Mono<CartEntity> deleteUserIdCartItems(String userId) {
        return cartRepository.findById(userId)
                .flatMap(existingCartItem->
                    cartRepository.delete(existingCartItem)
                            .then(Mono.just(existingCartItem)))
                .switchIfEmpty(Mono.error(new ResourceNotFoundException(HttpStatus.NOT_FOUND,
                        "CartItem for UserId" + userId + "not found")));
    }


    private CartDTO convertEntitytoDTO(CartEntity cartEntity){

        CartDTO cartDTO=new CartDTO();
        cartDTO.setUserId(cartEntity.getUserId());
        List<CartItemDTO> cartItemDtos= Optional.ofNullable(cartEntity.getCartItems()).orElse(new ArrayList<>())
                .stream().map(entity->{
                    CartItemDTO cartItemDTO=new CartItemDTO();
                    cartItemDTO.setProductRef(entity.getProductRef());
                    cartItemDTO.setQuantity(entity.getQuantity());
                    return cartItemDTO;})
                .collect(Collectors.toList());
        cartDTO.setCartItems(cartItemDtos);
        return cartDTO;
    }

    private CartEntity convertDTOtoEntity(CartDTO cartDTO){
        CartEntity cartEntity=new CartEntity();
        cartEntity.setUserId(cartDTO.getUserId());
        List<CartItemEntity> cartItemEntities=   Optional.ofNullable(cartDTO.getCartItems()).orElse(new ArrayList<>()).stream().map(dto->{
            CartItemEntity cartItemEntity=new com.nisum.ecomservice.model.CartItemEntity();
            cartItemEntity.setProductRef(dto.getProductRef());
            cartItemEntity.setQuantity(dto.getQuantity());
            return cartItemEntity;})
                .collect(Collectors.toList());

        cartEntity.setCartItems(cartItemEntities);
        return cartEntity;
    }
}