package com.nisum.ecomservice.service;

import com.nisum.ecomservice.config.AppConfig;
import com.nisum.ecomservice.dto.Order;
import com.nisum.ecomservice.dto.OrderItem;
import com.nisum.ecomservice.dto.OrderRequest;
import com.nisum.ecomservice.dto.OrderSubmission;
import com.nisum.ecomservice.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class OrderService {

    @Autowired
    ProductService productService;


    public Mono<Order> postOrder(OrderRequest orderRequest) {

        //create user object from user object details
        User user = orderRequest.getUser();
        Address address = orderRequest.getAddress();
        CartEntity cartEntity = orderRequest.getCartEntity();

        //for each item grab product details from products API
        List<Mono<Product>> productsToOrder = cartEntity.getCartItems().stream()
                .map(cartItem -> cartItem.getProductRef().getSku())
                .map(sku -> productService.getProductBySku(sku))

                .collect(Collectors.toList());

        Flux<Product> productFlux = Flux.concat(productsToOrder);

        Flux<OrderItem> orderItems = productFlux.map(product -> {
            OrderItem orderItem = new OrderItem();
            orderItem.setItemId(product.getSku());
            orderItem.setItemName(product.getProductName());
            orderItem.setItemSku(product.getSku());
            orderItem.setItemPrice(product.getPrice());

            List<CartItemEntity> items = cartEntity.getCartItems().stream().filter(cartItem -> cartItem.getProductRef().getSku()
                    .equals(product.getSku()))
                    .collect(Collectors.toList());

            Integer quantity = items.get(0).getQuantity();

            orderItem.setItemQuantity(quantity);

            return orderItem;
        });
       //

        Mono<Mono<Order>> map = orderItems.collectList().map(itemList -> {
            OrderSubmission orderSubmission = new OrderSubmission();
            orderSubmission.setOrderItems(itemList);
            orderSubmission.setCustomerAddress(address);
            orderSubmission.setCustomerId(user.getUserId());
            orderSubmission.setCustomerEmailAddress(user.getEmail());
            System.out.println(orderSubmission);
            return orderSubmission;
        }).map(orderSubmission -> postOrderToAPI(orderSubmission));


        //post order to OMS team
        //retrieve their response

        return map.flatMap(orderMono -> orderMono);

    }

    public Mono<Order> postOrderToAPI(OrderSubmission orderSubmission){
        return WebClient.create(String.format("%s/orders", AppConfig.getOrderManagementAPI()))
                .post()
                .bodyValue(orderSubmission)
                .retrieve()
                .bodyToMono(Order.class);
    }
}
/*
 .map(productMono -> {
                    productMono
                            .map(product -> {
                                OrderItem orderItem = new OrderItem();
                                orderItem.setItemId(product.getSku());
                                orderItem.setItemName(product.getProductName());
                                orderItem.setItemSku(product.getSku());
                                orderItem.setItemPrice(product.getPrice());

                                List<CartItem> items = user.getCart().stream().filter(cartItem -> cartItem.getProductRef().getSku()
                                        .equals(product.getSku()))
                                        .collect(Collectors.toList());
                                Integer quantity = items.get(0).getQuantity();

                                orderItem.setItemQuantity(quantity);
                                System.out.println(orderItem);
                                return  orderItem;
                            })
                })
 */